# Authors:
#   Petr Viktorin <pviktori@redhat.com>
#
# Copyright (C) 2013  Red Hat
# see file 'COPYING' for use and warranty information
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Nose plugin that generates "tcms-submit"-compatible "scripts"

Usage:
nosetests --collect-only --tcms-script=filename
"""

import os,sys
import unittest
import re
import xmlrpclib

import nose
from nose.plugins import Plugin

from ipapython.ipautil import shell_quote

TRAC_TICKET_RE = re.compile(r'fedorahosted.org/freeipa/ticket/(?P<id>[0-9]+)')
BUGZILLA_RE = re.compile(r'bugzilla.redhat.com/show_bug.cgi\?id=(?P<id>[0-9]+)')

class TCMSPlugin(Plugin):
    name = 'tcms-script'

    def __init__(self):
        super(TCMSPlugin, self).__init__()

    def options(self, parser, env=os.environ):
        parser.add_option(
            '--tcms-script', type='str', metavar='FILENAME',
            help='Write tcms-submit compatible "script" into this file')

    def configure(self, options, conf):
        if options.tcms_script:
            self.filename = options.tcms_script
            if not os.path.isabs(self.filename):
                raise ValueError('Absolute path needed for --tcms-script')
            self.enabled = True
            self.found = set()
            self.lines = [
                'echo "This is a file for tcms-submit, not a runnable script"',
                'exit',
                '']
            self.trac = xmlrpclib.ServerProxy(
                "https://fedorahosted.org/freeipa/xmlrpc") 
        else:
            self.enabled = False

    def report(self, stream):
        with open(self.filename, 'w') as file:
            print >>file, '\n'.join(self.lines)

    def startTest(self, test):
        # Add any test classes with context
        try:
            context = test.context
        except AttributeError:
            pass
        else:
            if isinstance(context, type):
                self.add_phase(context)

        # Un-wrap Nose test cases to get at the actual method
        while hasattr(test, 'test'):
            test = test.test

        # Add the test itself
        self.add_phase(test)


    def add_phase(self, item):
        """Add a phase (test case or context) to the output"""
        # The format is:
        # rlPhaseStartTest 'test_module-TestClass-test_name: Description [BZ#]'
        if item not in self.found:
            try:
                cls = item.im_class
                item_type = 'method'
            except AttributeError:
                cls = item
                item_type = 'class'
            name = '%s-%s' % (getattr(cls, '__module__', ''),
                              getattr(cls, '__name__', ''))
            if cls is not item:
                name += '-' + item.__name__
            try:
                docstring = item.__doc__
            except AttributeError:
                docstring = None
            if not docstring:
                description = 'Nose %s (no docstring)' % item_type
            else:
                description = docstring.strip().partition('\n')[0]
            name = name.replace('.', '-') 
            line = '%s: %s' % (name, description)
            for number in self.find_bugzilla_numbers(docstring or ''):
                line += ' [BZ%06d]' % number
            self.lines.append("rlPhaseStartTest %s" % shell_quote(line))
            self.found.add(item)

    def find_bugzilla_numbers(self, text):
        """Find Bugzilla bug numbers from text"""
        # Add RHBZ text from linked Trac tickets
        for trac_match in TRAC_TICKET_RE.finditer(text):
            trac_id = int(trac_match.group('id'))

            print >>sys.stderr, (
                'Finding Bugzilla for Trac ticket %s...' % trac_id)
            id, time_created, time_changed, attributes = self.trac.ticket.get(
                trac_id)
            bz_text = attributes['rhbz']
            text += '\n' + bz_text
            print >>sys.stderr, 'RHBZ field for %s: %s' % (trac_id, bz_text)

        # Find Bugzilla IDs
        numbers = set()
        for bz_match in BUGZILLA_RE.finditer(text):
            bz_number = int(bz_match.group('id'))

            numbers.add(bz_number)

        return sorted(numbers)
